#!/usr/bin/env python3
# -*- coding:utf-8 -*-

########################################
##      Radio Mania 2017
##      Experiment of "radiko.jp Dedicated Reciever"
##
##      Copyright 2017 Sansai Books
##      Programmed by, Nobuhide Tejima  
########################################

## Module Inport
import os
import signal
import subprocess
import RPi.GPIO as GPIO
import time
import smbus
import sys
import urllib.request
import xml.etree.ElementTree as ET

## I2C Setting
i2c = smbus.SMBus(1)
time.sleep(2)

## Software Version
software_version="1.003"

## GPIO Port Using
RED=8
GREEN=10
BACKLIGHT=12
ENC_A=16
ENC_B=18
CLICK=22

## GPIO Setting
GPIO.setmode( GPIO.BOARD )
GPIO.setup( RED, GPIO.OUT )
GPIO.setup( GREEN, GPIO.OUT )
GPIO.setup( BACKLIGHT, GPIO.OUT )
GPIO.setup( ENC_A, GPIO.IN, pull_up_down=GPIO.PUD_UP )
GPIO.setup( ENC_B, GPIO.IN, pull_up_down=GPIO.PUD_UP )
GPIO.setup( CLICK, GPIO.IN, pull_up_down=GPIO.PUD_UP )

## Config Setting
BLTIME=10
CLICK_TIME = 0.02
SHUTDOWN_MODE = 0  ## 0 = Program Close, 1 = Power OFF
SHUTDOWN_TIME = 3

## Initial value of Variable
BLON=0
CHANNEL=0
CHANNNEL_ONCE = 0
CHR=""
CLK = 0
CLK_LAST = 1
CLK_START = 0
PIECE=""
PIECE_ORDER=0
PIECE_CODE=""
ROTARY=0
ROTARY_PREVIOUS = ""

##LCD Initialize
def lcd_initialize():
        i2c.write_byte_data(0x50, 0x00, 0x01)
        time.sleep(0.4)
        i2c.write_byte_data(0x50, 0x00, 0x38)
        time.sleep(0.1)
        i2c.write_byte_data(0x50, 0x00, 0x0c)
        time.sleep(0.1)
        i2c.write_byte_data(0x50, 0x00, 0x06)
        time.sleep(0.1)

## LCD Display 
def lcd_display(charactor, line):
        if len(charactor) < 16:
                for i in range(16-len(charactor)):
                        charactor = charactor + " "
        if len(charactor) >16:
                charactor = charactor[0:16]
        if line == 0:
                i2c.write_byte_data(0x50, 0x00, 0x80)
                time.sleep(0.1)
        if line == 1:
                i2c.write_byte_data(0x50, 0x00, 0xc0)
                time.sleep(0.1)
        for j in range(len(charactor)): 
                PIECE = charactor [j]
                #print(PIECE ,ord(PIECE))
                i2c.write_byte_data(0x50, 0x80, ord(PIECE))
                time.sleep(0.01)

##LED Color
def led_color(color):
        if color =="OFF":
                GPIO.output( RED, False )
                GPIO.output( GREEN, False )
        if color =="RED":
                GPIO.output( RED, True )
                GPIO.output( GREEN, False )
        if color =="GREEN":
                GPIO.output( RED, False )
                GPIO.output( GREEN, True )
        if color == "ORANGE":
                GPIO.output( RED, True )
                GPIO.output( GREEN, True )
        
## Player Play
def player_play (STATION_NAME):
        cmd = "./play_radiko.sh "  + STATION_NAME
        proc = subprocess.Popen( cmd, shell=True )
        SP =  proc.pid
        #print( proc, SP )

## Player Kill
def player_kill():
        ##PID Detect
        contents = subprocess.check_output( "ps alx", shell=True )
        for l in contents.splitlines():
                        e = l.split()
                        check = e.count (b'mplayer')
                        if check > 0:
                                KILL_ID = int(e[2])
                                print ("***********",KILL_ID, "***********")
                                ## Player Killing
                                os.kill( KILL_ID, signal.SIGKILL )
                                print  (KILL_ID, "Killed!!" )

## Shutdown Sequence
def shutdown():
        lcd_display("Shut Down...",1)
        lcd_display(" ",0)
        time.sleep(3)
        lcd_initialize()
        GPIO.output (BACKLIGHT, False)
        led_color ("OFF")

## Operation
try:                                
        ## BOOT UP Sequence
        ## BOOT Display
        GPIO.output( BACKLIGHT, True )
        lcd_initialize()
        lcd_display("Initialize...",1)
        led_color("ORANGE")
        time.sleep(3)
        lcd_display("radiko.jp Radio",0)
        lcd_display("System Ver,"+software_version ,1)
        time.sleep(5)

        ## AREA Detect
        #AREA_CODE = JP14
        url = "https://radiko.jp/area"
        request = urllib.request.Request(url)
        response = urllib.request.urlopen(request)
        response_buffer = response.read().decode("utf-8").split('"')
        AREA_CODE = response_buffer[1]
        print(AREA_CODE)

        ## GET XML (from radiko.jp)
        url = "https://radiko.jp/v2/station/list/"+ AREA_CODE + ".xml"
        request = urllib.request.Request(url)
        response = urllib.request.urlopen(request)
        XmlData = response.read()

        STATION_TABLE = ET.fromstring(XmlData)
        print(STATION_TABLE.tag, STATION_TABLE.attrib)

        ## LOCATION DETECT
        AREA_TEMP = STATION_TABLE.attrib
        LOCATION_TEMP = "{area_name}".format(**AREA_TEMP).split()
        LOCATION = LOCATION_TEMP[0]
        print(LOCATION)

        ## ARRAY DEFINITION
        STATION_ID = []
        STATION_NAME = []

        ## STATION NUMBER COUNT
        STATION_COUNT = len(STATION_TABLE)
        for i in range(STATION_COUNT):
                STATION_ID.append(STATION_TABLE[i][0].text)
                STATION_NAME.append(STATION_TABLE[i][2].text)
        for j in range(STATION_COUNT):
                print("Station ID = ",j ,STATION_ID[j], STATION_NAME[j])
                    
        ## Nomal Running
        led_color("RED")
        lcd_display("[" + LOCATION + "]" ,0)

        while True:
                ## Capture GPIO Status
                STA0 = GPIO.input(ENC_A)
                STB0 = GPIO.input(ENC_B)
                CLK = GPIO.input(CLICK)
                ## Station Display
                if CHANNNEL_ONCE == 0:
                        lcd_display(STATION_NAME[CHANNEL],1)
                        CHANNNEL_ONCE = 1
                ## Back Light Control
                if time.time() - BLON > BLTIME:
                         GPIO.output(BACKLIGHT, False)
                ## Rotary Detect
                ROTARY = STA0 + STB0
                if ROTARY == 0:
                        if ROTARY_PREVIOUS == 1:
                                GPIO.output(BACKLIGHT, True)
                                BLON=time.time()
                                if CHANNEL <  STATION_COUNT - 1:
                                        CHANNEL = CHANNEL + 1
                                        STATION = STATION_NAME[CHANNEL]
                                        print (STATION)
                                        lcd_display(STATION_NAME[CHANNEL] ,1)
                                        time.sleep(0.1)
                        if ROTARY_PREVIOUS == 10:
                                GPIO.output(BACKLIGHT, True)
                                BLON=time.time()
                                if CHANNEL >  0:
                                        CHANNEL = CHANNEL - 1
                                        STATION = STATION_NAME[CHANNEL]
                                        print (STATION)
                                        lcd_display(STATION_NAME[CHANNEL] ,1)
                                        time.sleep(0.1)
                ROTARY_PREVIOUS = STA0 * 10 + STB0
                ## Enter Detect 
                NOW_TIME = time.time()
                if CLK == 0:
                        if CLK_LAST == 0 and NOW_TIME - CLK_START > SHUTDOWN_TIME:
                                GPIO.output(BACKLIGHT, True)
                                BLON=time.time()
                                print ("SHUT_DOWN!")
                                break
                        CLK_LAST = 0
                if CLK == 1:
                        if CLK_LAST == 0 and NOW_TIME - CLK_START > CLICK_TIME:
                                print ("CLICK!")
                                print (STATION_ID[CHANNEL] , STATION_NAME[CHANNEL] )
                                led_color("ORANGE")
                                player_kill()
                                GPIO.output(BACKLIGHT, True)
                                BLON=time.time()
                                player_play (STATION_ID[CHANNEL])
                                led_color("GREEN")
                                time.sleep(1)
                        #print (CHANNEL, STA0, STB0, ROTARY, ROTARY_PREVIOUS ,int(BLON))      
                        CLK_START  = time.time()
                        CLK_LAST = 1
                
except KeyboardInterrupt:
        print("Manual Close")
else:
        print("Normal Shut Down")
finally:
        player_kill()
        shutdown()
        GPIO.cleanup()
        if SHUTDOWN_MODE == 1:
                os.system("shutdown -h now")

